unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, VirtualTrees, FileUtil, Forms, Controls, Graphics, Dialogs,
  StdCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    bLoad: TButton;
    cbFilterBy: TComboBox;
    edFilter: TEdit;
    ImageList1: TImageList;
    VST: TVirtualStringTree;
    procedure bLoadClick(Sender: TObject);
    procedure cbFilterByChange(Sender: TObject);
    procedure edFilterChange(Sender: TObject);
    procedure VSTCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure VSTGetImageIndex(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Kind: TVTImageKind; Column: TColumnIndex; var Ghosted: Boolean;
      var ImageIndex: Integer);
    procedure VSTGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    procedure VSTHeaderClick(Sender: TVTHeader; Column: TColumnIndex;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    FSortCol: Integer;
    FSortDir: VirtualTrees.TSortDirection;
    procedure Filter(const ATyp: Integer; AText: String);
    procedure ResetFilter;
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

type
  PData = ^TData;
  TData = record
    FName: String;
    FOcupation: String;
    FYear: Integer;
    FMarried: Boolean;
    FImageIndex: Integer;
  end;

procedure TForm1.bLoadClick(Sender: TObject);
var
  Data: PData;
  Node: PVirtualNode;
begin
  VST.Clear;
  VST.NodeDataSize := SizeOf(TData);

  Node := VST.AddChild(nil);
  Data := VST.GetNodeData(Node);
  Data^.FName := 'erick';
  Data^.FOcupation := 'student';
  Data^.FYear := 18;
  Data^.FMarried := False;
  Data^.FImageIndex := 0;

  Node := VST.AddChild(nil);
  Data := VST.GetNodeData(Node);
  Data^.FName := 'magui';
  Data^.FOcupation := 'enfermera';
  Data^.FYear := 25;
  Data^.FMarried := True;
  Data^.FImageIndex := 1;

  Node := VST.AddChild(nil);
  Data := VST.GetNodeData(Node);
  Data^.FName := 'leo';
  Data^.FOcupation := 'ingenier';
  Data^.FYear := 30;
  Data^.FMarried := True;
  Data^.FImageIndex := 2;

  FSortCol := 1;
  FSortDir := VirtualTrees.sdAscending;
  VST.SortTree(1, VirtualTrees.sdAscending);
end;

procedure TForm1.cbFilterByChange(Sender: TObject);
begin
  edFilter.Text := '';
  edFilter.SetFocus;
end;

procedure TForm1.VSTGetImageIndex(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Kind: TVTImageKind; Column: TColumnIndex; var Ghosted: Boolean;
  var ImageIndex: Integer);
var
  Data: PData;
begin
  if Column = 0 then
  begin
    Data := VST.GetNodeData(Node);
    ImageIndex := Data^.FImageIndex;
  end;
end;

procedure TForm1.VSTGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
var
  Data: PData;
begin
  Data := VST.GetNodeData(Node);
  case Column of
    0: CellText := '';
    1: CellText := Data^.FName;
    2: CellText := Data^.FOcupation;
    3: CellText := IntToStr(Data^.FYear);
    4: CellText := BoolToStr(Data^.FMarried, True);
  end;
end;

procedure TForm1.VSTHeaderClick(Sender: TVTHeader; Column: TColumnIndex;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Column = 0) then
    Exit;
  if Button = mbLeft then
  begin
    with Sender, Treeview do
    begin
      if (SortColumn = NoColumn) or (SortColumn <> Column) then
      begin
        SortColumn    := Column;
        SortDirection := VirtualTrees.sdAscending;
      end
      else
      begin
        if SortDirection = VirtualTrees.sdAscending then
          SortDirection := VirtualTrees.sdDescending
        else
          SortDirection := VirtualTrees.sdAscending;
        FSortDir := SortDirection;
      end;
      SortTree(SortColumn, SortDirection, False);
      FSortCol := Sender.SortColumn;
    end;
  end;
end;

procedure TForm1.VSTCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
var
  Data1: PData;
  Data2: PData;
begin
  Data1 := VST.GetNodeData(Node1);
  Data2 := VST.GetNodeData(Node2);
  case Column of
    1: Result := CompareText(Data1^.FName, Data2^.FName);
    2: Result := CompareText(Data1^.FOcupation, Data2^.FOcupation);
    3: Result := Data1^.FYear - Data2^.FYear;
    4: Result := CompareText(UpperCase(BoolToStr(Data1^.FMarried)), UpperCase(BoolToStr(Data2^.FMarried)))
  end;
end;

procedure TForm1.Filter(const ATyp: Integer; AText: String);
var
  Node: PVirtualNode;
  Data: PData;
  Str: String;
begin
  Node := VST.GetFirst;
  while Assigned(Node) do
  begin
    Data := VST.GetNodeData(Node);
    case ATyp of
      1: Str := Data^.FName;
      2: Str := Data^.FOcupation;
      3: Str := IntToStr(Data^.FYear);
      4: Str := BoolToStr(Data^.FMarried, True);
    end;
    if ATyp = 0 then
      VST.IsVisible[Node] := (Pos(UpperCase(AText), UpperCase(Data^.FName)) > 0) or
                             (Pos(UpperCase(AText), UpperCase(Data^.FOcupation)) > 0) or
                             (Pos(UpperCase(AText), UpperCase(IntToStr(Data^.FYear))) > 0) or
                             (Pos(UpperCase(AText), UpperCase(BoolToStr(Data^.FMarried, True))) > 0)
    else
      VST.IsVisible[Node] := Pos(UpperCase(AText), UpperCase(Str)) > 0;
    Node := VST.GetNext(Node);
  end;
end;

procedure TForm1.ResetFilter;
var
  Node: PVirtualNode;
begin
  Node := VST.GetFirst;
  while Assigned(Node) do
  begin
    VST.IsVisible[Node] := True;
    Node := VST.GetNext(Node);
  end;
end;

procedure TForm1.edFilterChange(Sender: TObject);
begin
  if edFilter.Text = '' then
    ResetFilter
  else
    Filter(cbFilterBy.ItemIndex, edFilter.Text);
end;

end.

